;Infrarotempfnger mit Tiny2313 fr Pollin-IR-Bausatz

;Spezialversion fr Michael Niemyt zum Aufrsten einer RFT-Fono-Anlage.

;Durch Auswechseln des Pollin-AVRs gegen einen Tiny2313 mit diesem Programm
;entsteht ein vllig neuer IR-Empfnger, der mit derselben Fernbedienung
;angesprochen werden kann.


.include "tn2313def.inc"    ;Namen benannter Bytes und Bits bekannt machen

;Vereinbarte Konstanten:
.equ clock=1000000  ;1MHz Taktfrequenz
.equ signal=pd2     ;Signaleingang
.equ led1=pd3       ;Empfangs-LED-Ausgang
.equ shift0=0b1111111111100000  ;Bitmuster fr Empfangs-Start
.equ timout=50      ;Timeout fr Impulsabstand
.equ adr0=7         ;Adresse der Fernbedienung (7)
.equ dauero=125     ;Dauer fr Optokoppler je 4ms (500ms)
.equ dauerm=40      ;Dauer fr Motorpoti je 4ms (160ms)

;Variablen in SRAM-Zellen:
.dseg               ;SRAM
.org $60            ;ab Adresse 196 (nach mapped-I/O-Bereich)
dauerz:  .byte 10   ;Ausschalt-Zhler fr die Relais

;Variablen in Registern (untere Register)
.def srsk=r2        ;SREG-Kopie
.def null=r3        ;immer 0
.def shiftl=r4      ;Empfangsregister L
.def shifth=r5      ;Empfangsregister H
.def zsalt=r6       ;alter Zeitstempel
.def zaehler=r7     ;freilaufender Software-Timer 256 s
.def poko=r8        ;Port-Kopie vom Eingang
.def tout=r9        ;Timeout Impulsabstand
.def cmdalt=r10     ;zuletzt empfangenes Kommando (fr Entprellung)
.def cmd=r11        ;Kommando-Wert nach RECS80-Norm
.def afcz=r12       ;AFC-Zhler (Feinabstimmung)

;Variablen in Registern (obere Register)
.def flags=r16      ;einige Flags bzw. Merker
    .equ zovf=7         ;Zhler-berlauf fr Zeitstempel
    .equ iovf=6         ;Impulsbreite-berlauf
    .equ neu=5          ;neues Telegramm eingetroffen
    .equ repeat=4       ;Telegramm entspricht dem Vorhergehendem

.def tas=r17        ;Tasten-Wert nach eigener Norm
.def dauer=r18      ;Dauer des Schaltvorgangs am Port
.def wl=r24         ;Working L
.def wh=r25         ;Working H

;Beginn des Programmspeichers, reservierte Adressen fr Reset und Interrupts:
.cseg               ;Programmspeicher
.org 0              ;ab Adresse 0
 rjmp RESET         ;Reset Handler
 rjmp exint0        ;External Interrupt0 Handler
 rjmp nix;INT1          ;External Interrupt1 Handler
 rjmp nix;TIM1_CAPT     ;Timer1 Capture Handler
 rjmp nix;TIM1_COMPA    ;Timer1 CompareA Handler
 rjmp nix;TIM1_OVF      ;Timer1 Overflow Handler
 rjmp TIM0_OVF      ;Timer0 Overflow Handler
 rjmp nix;USART0_RXC    ;USART0 RX Complete Handler
 rjmp nix;USART0_DRE    ;USART0,UDR Empty Handler
 rjmp nix;USART0_TXC    ;USART0 TX Complete Handler
 rjmp nix;ANA_COMP      ;Analog Comparator Handler
 rjmp nix;PCINT         ;Pin Change Interrupt
 rjmp nix;TIMER1_COMPB  ;Timer1 Compare B Handler
 rjmp nix;TIMER0_COMPA  ;Timer0 Compare A Handler
 rjmp nix;TIMER0_COMPB  ;Timer0 Compare B Handler
 rjmp nix;USI_START     ;USI Start Handler
 rjmp nix;USI_OVERFLOW  ;USI Overflow Handler
 rjmp nix;EE_READY      ;EEPROM Ready Handler
 rjmp nix;WDT_OVERFLOW  ;Watchdog Overflow Handler

nix:                    ;eine Falle fr unerlaubte Interrupts (Prog-Fehler)
 rjmp nix                   ;Falle...
; reti                       ;zurck...

reset:                  ;Initialisierung
 ldi wl,low(ramend)         ;Stackpointer
 out spl,wl                 ;initialisieren
 clr null                   ;immer 0
 clr afcz                   ;Zhler Feinabstimmung auf 0
 ldi wl,$ff                 ;alle Pins von
 out ddrb,wl                ;PortB als Ausgang
 out portb,null             ;und L-Pegel
 ldi wl,0b10000111          ;PullUps fr Eingnge
 out portd,wl               ;einschalten (auer LED)
 out porta,wl               ;auch PortA
 com wl                     ;LEDs und Relais 9 und 10 als Ausgang
 out ddrd,wl                ;Jumper und IR-Sensor als Eingang
 out ddra,null              ;PortA (Jumper) als Eingang
 
 out tccr0a,null            ;keine Sonderfunktionen
 ldi wl,1                   ;Vorteiler 1:1
 out tccr0b,wl              ;fr Timer0
 ldi wl,1<<toie0            ;OVF0-Interrupt
 out timsk,wl               ;einschalten
 ldi wl,(1<<se)|(1<<isc00)  ;Sleep-Mode Idle und wechselnde Flanke
 out mcucr,wl               ;fr Int0 vorwhlen
 ldi wl,1<<int0             ;Int0
 out gimsk,wl               ;aktivieren
 sei                        ;Interrupts freigeben
 clr tas                    ;ungltige Taste

mainloop:               ;Hauptschleife
 sbrc flags,zovf            ;Port nach gewisser Zeit
 rcall machaus              ;abschalten
 sbrc flags,neu             ;neues Telegramm eingetroffen? - nein
 rcall neutelda             ;ja, behandeln...
 sleep                      ;bis zum nchsten Interrupt pennen
 rjmp mainloop              ;nochmal...

machaus:                ;Relais wieder ausschalten
 cbr flags,1<<zovf          ;Jobflag lschen
 ldi yl,low(dauerz)         ;Y-Pointer auf Adresse des
 ldi yh,high(dauerz)        ;ersten Ausschalt-Timers
 ldi zl,low(reltab*2)       ;Pointer auf
 ldi zh,high(reltab*2)      ;Relais-Bitmuster
machaus00:
 lpm r0,z+                  ;Bitmuster fr PortB
 lpm r1,z+                  ;und PortD holen
 com r0                     ;Bitmuster
 com r1                     ;invertieren
 ld wl,y                    ;Zhlerstand holen (Pointer nicht verndern)
 tst wl                     ;wenn schon 0,
 breq machaus01             ;dann fertig...
 dec wl                     ;Timeout Relais runter
 brne machaus01             ;schon unten? - nein...
 in wh,portb                ;PortB einlesen
 and wh,r0                  ;auszuschaltendes Bit lschen
 out portb,wh               ;zurckschreiben
 in wh,portd                ;PortD einlesen
 and wh,r1                  ;auszuschaltendes Bit lschen
 out portd,wh               ;zurckschreiben
machaus01:
 st y+,wl                   ;Timer zurckschreiben und Pointer erhhen
 cpi yl,dauerz+10           ;alle 10 Zhler behandelt?
 brne machaus00             ;nein, nochmal...
 ret                        ;ja, fertig...


neutelda:               ;neues Telegramm eingetroffen
 cbr flags,1<<neu           ;Jobflag lschen
 mov tas,cmd                ;Kopie der Kommando-Nummer
 andi tas,63                ;Toggelbit ausblenden
; in 'tas' steht jetzt der Tastencode ohne Toggelbit
 ldi zl,low(tastab*2)       ;Z-Pointer auf Beginn der Tabelle mit 
 ldi zh,high(tastab*2)      ;Relais-Nummern im Flash positionieren
 add zl,tas                 ;empfangenen Tastencode als Index auf Tabelle
 adc zh,null                ;mit bertrag addieren
 lpm tas,z                  ;Tabellenwert (Relais-Nummer) nach tas holen
 sbrc tas,6                 ;ist Relais-Nummer gltig (<64) - ja...
 rjmp neuteldaend           ;nein, nicht behandeln...
 ;die Relais-Nummer ist nicht 64..95, also gltig
 ldi wl,dauerm              ;erstmal Motorpoti-Abschalttermin vermuten
 sbrc tas,3                 ;Motorpoti? - nein, Optokoppler...
 rjmp mopo                  ;ja, immer behandeln.., 
opto:
 sbrc flags,repeat          ;ist das Telegramm eine Wiederholung? - nein...
 rjmp neuteldaend           ;ja, nicht behandeln...
 ldi wl, dauero             ;Optokoppler-Abschalttermin setzen
mopo:
 ldi yl,low(dauerz)         ;Y-Pointer auf Beginn Array mit
 ldi yh,high(dauerz)        ;Timeout-Zhlern im SRAM
 add yl,tas                 ;Relais-Nummer als Index dazu
 st y,wl                    ;Abschaltzhler auf Abschalttermin setzen
 ;der Abschalttermin ist nun gesetzt
 ldi zl,low(reltab*2)       ;Z-Pointer auf Beginn der Tabelle mit 
 ldi zh,high(reltab*2)      ;Relais-Bitmustern im Flash positionieren
 lsl tas                    ;Rel-Nr. mal 2, weil jeder Eintrag 2 Bytes hat
 add zl,tas                 ;und als Index auf Tabelle
 adc zh,null                ;mit bertrag addieren
 lsr tas                    ;Rel.-Nr. wieder korregieren
 ;der Z-Pointer zeigt auf das Bytepaar mit Bitmustern zum Einschalten des
 ;Relais
 in wl,portb                ;jetzigen Portzustand PortB holen,
 lpm r0,z+                  ;Bitmuster fr PortB aus Flash-Tabelle holen
 or wl,r0                   ;Bit des Relais setzen
 out portb,wl               ;PortB zurckschreiben
 in wl,portd                ;jetzigen Portzustand PortD holen,
 lpm r0,z+                  ;Bitmuster fr PortD aus Flash-Tabelle holen
 or wl,r0                   ;Bit des Relais setzen
 out portd,wl               ;PortD zurckschreiben
 ;das zur Taste gehrige Bitmuster ist nun an den Ports ausgegeben
neuteldaend:            ;Ende der Tastenauswertung
 clr tas                    ;Tastenwert lschen
 ret                        ;fertig


TIM0_OVF:               ;ISR alle 256 Takte (256 s)
 in srsk,sreg               ;SREG sichern
 inc zaehler                ;Softwaretimer erhhen
 mov xl,zaehler             ;Kopie zum Maskieren
 andi xl,15                 ;jedes 16. mal (4ms)
 brne tim0_ovf1             ;jetzt nicht
 sbr flags,1<<zovf          ;dem Hauptprogramm Zeittakt mitteilen
tim0_ovf1:
 cp zaehler,tout            ;Timeout Impulsabstand erreicht?
 brne isrend                ;nein...
imperr:                 ;Impulserror, neues Telegramm vorbereiten
 mov zsalt,zaehler          ;Zeitstempel holen
 ldi xl,timout              ;Timeout
 add xl,zsalt               ;berechnen und
 out ocr0a,xl               ;setzen
 ldi xl,low(shift0)         ;Start-
 mov shiftl,xl              ;Bitmuster
 ldi xl,high(shift0)        ;ins Shiftregister
 mov shifth,xl              ;schreiben
isrend:
 out sreg,srsk              ;SREG wiederherstellen
 reti                       ;fertig...
 
exint0:                 ;ISR Impuls vom Empfnger
 in srsk,sreg               ;SREG sichern
 in poko,pind               ;Kopie vom Eingangsport zur Strunterdrckung
 sbrc poko,signal           ;fallende Flanke? - ja...
 rjmp exint01               ;nein, steigende Flanke...
 sbi portd,led1             ;RX-LED ein
 rjmp isrend                ;fertig
exint01:
 cbi portd,led1             ;ja, RX-LED aus
 mov xl,zaehler             ;ja, Impuls gltig, aktuellen Zeitstempel holen
 sub xl,zsalt               ;Differenz zum vorigen Zeitstempel
 mov zsalt,zaehler          ;neuen Zeitstempel merken
 ldi wh,timout              ;Timeout fr Impulsabstanderror 
 mov tout,zaehler           ;auf "jetzt+x" 
 add tout,wh                ;setzen
 cpi xl,17                  ;Impulsabstand unter 0-Wert? (18..22)
 brcs imperr                ;ja, unter 18, Fehler...
 cpi xl,23                  ;Impulsabstand noch 0-Wert? (18..22)
 brcc exint02               ;nein, ber 22, kein 0-Bit mehr
 clc                        ;ja, 0-Bit, Carry lschen
 rjmp shiftin               ;und einschieben
exint02:
 cpi xl,27                  ;Impulsabstand unter 1-Wert? (28..32)
 brcs imperr                ;ja, unter 28, Fehler...
 cpi xl,33                  ;Impulsabstand noch 1-Wert? (28..32)
 brcc imperr                ;nein, ber 32, kein 1-Bit mehr, Error
 sec                        ;ja, 1-Bit, Carry setzen

shiftin:                ;schiebt Carry ein
 rol shiftl                 ;Carry ins untere Shiftregister schieben
 rol shifth                 ;berlauf ins obere Shiftregister schieben
 sbrc shifth,7              ;komplettes Telegramm eingelesen? - ja...
 rjmp isrend                ;nein, fertig...
;Telegramm ist komplett eingelesen
 lsl shiftl                 ;Adressbits
 rol shifth                 ;nach
 lsl shiftl                 ;oben
 rol shifth                 ;schieben,
 lsr shiftl                 ;Command wieder
 lsr shiftl                 ;zurck
 bst shifth,3               ;Togglebit in
 bld shiftl,6               ;Command kopieren
 ldi xl,7                   ;nur die 
 and shifth,xl              ;Adressbits maskieren
 ldi xl,adr0                ;zum Vergleich
 cp shifth,xl               ;stimmt die Adresse?
 breq shiftin1              ;ja...
 rjmp imperr                ;nein, verwerfen...
shiftin1:               ;Adresse stimmt
 cbr flags,1<<repeat        ;Wiederholflag erstmal lschen
 cp shiftl,cmd              ;dieses Telegramm schon erkannt?
 breq shiftin1a             ;ja, weiter...
 cp shiftl,cmdalt           ;nein, empfangenes Telegramm stabil?
 breq shiftin2              ;ja...
 mov cmdalt,shiftl          ;nein, neues Telegramm merken,
 rjmp imperr                ;neuen Empfang vorbereiten...
shiftin1a:
 sbr flags,1<<repeat        ;Wiederholflag setzen
shiftin2:               ;Telegramm stimmt mit vorherigem berein
 mov cmd,shiftl             ;Kommandowert bernehmen
 sbr flags,1<<neu           ;neues Telegramm melden
 rjmp imperr                ;nchstes Telegramm vorbereiten...

tastab:     ;Tabelle mit Relais-Nummern (0..9) zu den benutzten Tasten
;als Index dient der empfangene Tastencode, unbenutzte Codes erhalten 64

.db 64,64,64,64,64,64,64,64 ;00..07
.db 64,64,64,64,64,64,64,64 ;08..15
.db 64,64,64,64,64,64,64,64 ;16..23
.db 64,64,64,64,64,64,64,64 ;24..31
.db  2, 6, 8, 9, 1, 5, 0, 4 ;32..39 Tastencodes 32..39 benutzt
.db  3, 7, 3, 7,64,64,64,64 ;40..47 40..41 = RCS9082, 42..43 = TV400
.db 64,64,64,64,64,64,64,64 ;48..55
.db 64,64,64,64,64,64,64,64 ;56..63

;Die Tastencodes sind mechanisch wie folgt angeordnet:
;Handsender TV400:
; 17  18  06  62  63
; 19  20  04  05  15
; 21  22  56  26  27
; 23  24  52  30  28
; 25  16  48  31  29
;
; 38  36  32  42  34
; 39  37  33  43  35
;
; 50      51      49

;Handsender RCS9082:
; 17  18  06  62  63
; 19  20  04  05  15
; 21  22  56  26  27
; 23  24  52  30  28
; 25  16  48  31  29
;
; 38  36  32  40  34
; 39  37  33  41  35
;
; 50  55  51  53  49


reltab:     ;Tabelle mit Bitmustern fr die Relais (10 Eintrge)
.db   1,  0     ;Bitmuster fr PortB und PortD
.db   2,  0
.db   4,  0
.db   8,  0
.db  16,  0
.db  32,  0
.db  64,  0
.db 128,  0
.db   0, 32
.db   0, 64

