;Infrarotempfnger mit Tiny2313 fr Pollin-IR-Bausatz

;Spezialversion fr den Einsatz z.B. als Rolladen-Steuerung.


;Durch Auswechseln des Pollin-AVRs gegen einen Tiny2313 mit diesem Programm
;entsteht ein vllig neuer IR-Empfnger, der mit derselben Fernbedienung
;angesprochen werden kann.

;Dabei stehen neben den vom Bausatz bereitgestellten Ausgngen alle vom
;Bausatz nicht genutzten Portpins des ATTiny2313 zur Verfgung.


.include "tn2313def.inc"
;feste Konstanten:
.equ clock=1000000  ;1MHz Taktfrequenz
.equ signal=pd2     ;Signaleingang
.equ led1=pd3       ;Empfangs-LED-Ausgang
.equ shift0=0b1111111111100000  ;Bitmuster fr Empfangs-Start
.equ timout=50      ;Timeout fr Impulsabstand
.equ adr0=7         ;Adresse der Fernbedienung (7)

;vom Benutzer manipulierbare Konstanten:

;Zhlerstand (Anzahl gleicher Telegramme+1) kurzer Tastendruck (Entprellung)
.equ takurz=1       ;sollte 1 sein, bei vielen Strungen auch 2

;Zhlerstand (Anzahl gleicher Telegramme+1) langer Tastendruck (Repeat)
.equ talang=8       ;Wert sollte zwischen 5 und 15 liegen

;Rolladen-Timeout kurz (1/15 Sekunde je Wert)
.equ tokurz=6       ;ist von "takurz" abhngig, 5..10 ist ok

;Rolladen-Timeout lang (16 Sekunden je Wert)
.equ tolang=1       ;1 (16s) fr "Trockentest", 8..20 fr prakt. Einsatz


.dseg               ;SRAM
.org $60            ;ab Adresse 96 (nach mapped-I/O-Bereich)
dauerz:     .byte 80;40 Ausschalt-Zhler fr die el. Relais

                    ;r0=Aus-Maske A
                    ;r1=Ein-Maske A
.def srsk=r2        ;SREG-Kopie
.def null=r3        ;immer 0
.def shiftl=r4      ;Empfangsregister L
.def shifth=r5      ;Empfangsregister H
.def zsalt=r6       ;alter Zeitstempel
.def zaehler=r7     ;freilaufender Software-Timer 256 s
.def poko=r8        ;Port-Kopie vom Eingang
.def tout=r9        ;Timeout Impulsabstand
.def cmd=r10        ;Kommando-Wert nach RECS80-Norm
.def ausmskb=r11    ;Aus-Maske B
.def ausmskd=r12    ;Aus-Maske D+A
.def einmskb=r13    ;Ein-Maske B
.def einmskd=r14    ;Ein-Maske D+A

.def flags=r16      ;einige Flags (Boolean-Variablen)
    .equ zovf=7         ;Zhler-berlauf fr Zeitstempel
    .equ iovf=6         ;Impulsbreite-berlauf
    .equ neu=5          ;neues Telegramm eingetroffen
    .equ repeat=4       ;Tasten-Dauerdruck liegt an
    .equ toaktiv=3      ;Timeout ist aktiv

.def tas=r17        ;Tasten-Wert nach eigener Norm
.def dauer=r18      ;Dauer des Schaltvorgangs am Port
.def kanal=r19      ;momentan aktive Kanalnummer
.def telez=r20      ;Telegrammzhler (Entprellung/Repeat)
.def vtsek=r21      ;Vorteiler fr Sekunde

.def wl=r24         ;Working L
.def wh=r25         ;Working H


.cseg               ;Programmspeicher
.org 0              ;ab Adresse 0
 rjmp RESET         ;Reset Handler
 rjmp exint0        ;External Interrupt0 Handler
 rjmp nix;INT1          ;External Interrupt1 Handler
 rjmp nix;TIM1_CAPT     ;Timer1 Capture Handler
 rjmp nix;TIM1_COMPA    ;Timer1 CompareA Handler
 rjmp nix;TIM1_OVF      ;Timer1 Overflow Handler
 rjmp TIM0_OVF      ;Timer0 Overflow Handler
 rjmp nix;USART0_RXC    ;USART0 RX Complete Handler
 rjmp nix;USART0_DRE    ;USART0,UDR Empty Handler
 rjmp nix;USART0_TXC    ;USART0 TX Complete Handler
 rjmp nix;ANA_COMP      ;Analog Comparator Handler
 rjmp nix;PCINT         ;Pin Change Interrupt
 rjmp nix;TIMER1_COMPB  ;Timer1 Compare B Handler
 rjmp nix;TIMER0_COMPA  ;Timer0 Compare A Handler
 rjmp nix;TIMER0_COMPB  ;Timer0 Compare B Handler
 rjmp nix;USI_START     ;USI Start Handler
 rjmp nix;USI_OVERFLOW  ;USI Overflow Handler
 rjmp nix;EE_READY      ;EEPROM Ready Handler
 rjmp nix;WDT_OVERFLOW  ;Watchdog Overflow Handler
nix:                    ;unerlaubte Interrupts
 rjmp nix                   ;Falle...
 reti                       ;zurck...


reset:                  ;Initialisierung
 ldi wl,low(ramend)         ;Stackpointer
 out spl,wl                 ;initialisieren
 clr null                   ;immer 0
 ldi wl,$ff                 ;alle Pins von
 out ddrb,wl                ;PortB als Ausgang
 out ddra,wl                ;PortA auch
 out portb,null             ;und L-Pegel
 out porta,null             ;und L-Pegel
 ldi wl,0b00000100          ;PullUp fr IR-Eingang
 out portd,wl               ;einschalten, alle anderen Ports
 ldi wl,0b11111011          ;alles auer IR-Eingang 
 out ddrd,wl                ;als Ausgang
 ldi yl,low(dauerz)
 ldi yh,high(dauerz)
reset0:                     ;Timeout-Array lschen
 st y+,null
 cpi yl,low(dauerz+64)
 brne reset0
 out tccr0a,null            ;keine Sonderfunktionen
 ldi wl,1                   ;Vorteiler 1:1
 out tccr0b,wl              ;fr Timer0
 ldi wl,1<<toie0            ;OVF0-Interrupt
 out timsk,wl               ;einschalten
 ldi wl,(1<<se)|(1<<isc00)  ;Sleep-Mode Idle und wechselnde Flanke
 out mcucr,wl               ;fr Int0 vorwhlen
 ldi wl,1<<int0             ;Int0
 out gimsk,wl               ;aktivieren
 ldi flags,0                ;definierte 
 ldi tas,0                  ;Startbedingungen
 out portb,null             ;alle Relais ausschalten
 sei                        ;Interrupts freigeben

mainloop:               ;Hauptschleife
 sbrc flags,zovf            ;Port nach gewisser Zeit
 rjmp machaus               ;abschalten
 sbrc flags,neu             ;neues Telegramm eingetroffen? - nein
 rjmp neutelda              ;ja, behandeln...
 sleep                      ;bis zum nchsten Interrupt pennen
 rjmp mainloop              ;nochmal...


holmaske:               ;UP, holt Masken aus Flash, Index in tas
 ldi zl,low(reltab*2)       ;Z-Pointer fr LPM
 ldi zh,high(reltab*2)      ;auf Tabelle mit Schalt-Bitmustern
 mov r0,tas                 ;Wert sichern (statt Push)
 lsl tas                    ;Tastennummer mal 4, weil Tabelle
 lsl tas                    ;4 Bytes je Eintrag hat
 add zl,tas                 ;Index mit
 adc zh,null                ;bertrag addieren
 mov tas,r0                 ;Wert wiederherstellen (statt Pop)
 ;Z-Pointer steht jetzt auf indiziertem Datensatz
 lpm ausmskd,z+             ;Aus-Maske Port D+A aus Flash-Tabelle holen
 lpm einmskd,z+             ;Ein-Maske Port D+A aus Flash-Tabelle holen
 lpm ausmskb,z+             ;Aus-Maske Port B aus Flash-Tabelle holen
 lpm einmskb,z+             ;Ein-Maske Port B aus Flash-Tabelle holen
 clr r0                     ;r0 (Aus-Maske PortA) erstmal auf 0
 clr r1                     ;r1 (Ein-Maske PortA) erstmal auf 0
 bst ausmskd,2              ;Aus-Maskenbits 
 bld r0,1                   ;des PortA
 bst ausmskd,3              ;nach r0
 bld r0,0                   ;kopieren
 bst einmskd,2              ;Ein-Maskenbits 
 bld r1,1                   ;des PortA
 bst einmskd,3              ;nach r1
 bld r1,0                   ;kopieren
 ldi wl,0b11110011          ;A-Bits
 and ausmskd,wl             ;der D-Masken
 and einmskd,wl             ;lschen
 com r0                     ;Aus-Maske A invertieren
 com ausmskd                ;Aus-Maske D invertieren
 com ausmskb                ;Aus-Maske B invertieren
 ret                        ;fertig und zurck...


toggelmode:             ;keine Rolladentaste
 sbrc flags,repeat          ;langer Tastendruck? - nein, kurzer...
 rjmp mainloop              ;ja, nicht behandeln, fertig...
 out pina,r1                ;Portpins invertieren,
 out pinb,einmskb           ;falls entsprechendes Bit
 out pind,einmskd           ;in Ein-Maske gesetzt war
 rjmp mainloop              ;fertig...


setkurz:                ;setzt Timeout auf kurz (macht indirekt aus)
 ;Y-Pointer ist bereits positioniert
 ldi wl,tokurz              ;kurzer Timeout 
 std y+0,wl                 ;setzen
 std y+1,null               ;langen Timeout lschen
 rjmp mainloop              ;fertig...


neutelda:               ;neues Telegramm eingetroffen
 cbr flags,1<<neu           ;Jobflag lschen
 ldi zl,low(tastab*2)       ;Z-Pointer auf Beginn der Tabelle mit 
 ldi zh,high(tastab*2)      ;eigenen Tastenwerten im Flash
 mov wl,cmd                 ;empfangenen Wert holen
 andi wl,63                 ;Toggle-Bit lschen
 add zl,wl                  ;und als Index auf Tabelle
 adc zh,null                ;mit bertrag addieren
 lpm tas,z                  ;Tabellenwert (Tasten-Nummer) nach tas holen
 ;erstmal vorsorglich den Y-Pointer auf Timeout-Zhler positionieren:
 ldi yl,low(dauerz)         ;nein, Y-Pointer auf Beginn Array mit
 ldi yh,high(dauerz)        ;Timeout-Zhlern im SRAM
 mov wl,tas                 ;Kopie der Tasten-Nummer
 cbr wl,1<<0                ;Bit 0 lschen (nur gerade Zahlen als Tastenpaar)
 add yl,wl                  ;Tasten-Paar-Nummer 
 adc yh,null                ;als Index addieren 
 ;der Y-Pointer zeigt jetzt auf den zum Tastenpaar gehrenden Timeout-Zhler
 rcall holmaske             ;holt die Ein- und Aus-Masken der Taste "tas"
 ;Aus-Maske D enthlt in Bit7 das Flag fr Togglemode (kein Rolladenbetrieb)
 sbrs ausmskd,7             ;ist das Toggel-Mode-Bit gesetzt? - nein...
 rjmp toggelmode            ;ja, separate Abarbeitung...

 ;es wurde eine Rolladensteuertaste bettigt
 sbrc flags,repeat          ;langer Tastendruck? - nein...
 rjmp setkurz               ;ja, kurzen Timeout nachtriggern...
 ;es war also ein kurzer Tastendruck...
 ldd wl,y+0                 ;L-Byte Timeout-Zhler holen
 ldd wh,y+1                 ;H-Byte Timeout-Zhler holen
 adiw wh:wl,0               ;ist der Timeout aktiv (! 0)?
 brne setkurz               ;ja, ber kurzes Timeout indirekt ausschalten
 ;kurzer Tastendruck, Timeout war nicht aktiv, also Einschalt-Tastendruck...
 ;langen Timeout setzen (Fahrt auf Anschlag)
 ldi wl,tokurz              ;kurzer Timeout ist auch das L-Byte vom
 std y+0,wl                 ;langen Timeout, also auch setzen
 ldi wl,tolang              ;langen Timeout ins
 std y+1,wl                 ;H-Byte vom Timeout setzen
 ;Portpins setzen, Gegenrichtung aus
 in wl,porta                ;Port A
 and wl,r0                  ;eigene Richtung und Gegenrichtung lschen
 or wl,r1                   ;eigene Richtung setzen
 out porta,wl               ;und zurckschreiben
 in wl,portb                ;Port B
 and wl,ausmskb             ;eigene Richtung und Gegenrichtung lschen
 or wl,einmskb              ;eigene Richtung setzen
 out portb,wl               ;und zurckschreiben
 in wl,portd                ;Port D
 and wl,ausmskd             ;eigene Richtung und Gegenrichtung lschen
 or wl,einmskd              ;eigene Richtung setzen
 out portd,wl               ;und zurckschreiben
 rjmp mainloop              ;fertig...


machaus:                ;Job der Mainloop (alle 65,536ms)
 cbr flags,1<<zovf          ;Jobflag lschen
 ldi yl,low(dauerz-2)       ;Y-Pointer auf einen Schritt vor Array 
 ldi yh,high(dauerz-2)      ;mit Timeout-Zhler setzen
 ldi tas,-2                 ;Tastennummer einen Schritt vor 0
machaus1:
 adiw yh:yl,2               ;Index erhhen
 subi tas,-2                ;Tastennummer um 2 erhhen
 cpi tas,40                 ;letztes Tastenpaar erreicht?
 brne machaus2              ;nein, weiter...
 rjmp mainloop              ;ja, fertig...

machaus2:
 ldd wl,y+0                 ;L-Byte holen
 ldd wh,y+1                 ;H-Byte holen
 sbiw wh:wl,1               ;Zhler runter
 brcs machaus1              ;Unterlauf? - ja, Timeout nicht aktiv, nchster...
 std y+0,wl                 ;nein, verminderten Zhler
 std y+1,wh                 ;zurckschreiben
 brne machaus1              ;Zhler abgelaufen? - nein, nchster Zhler...
 ;dieser Punkt wird nur erreicht, wenn ein Timeout-Zhler abgelaufen ist
 ;in tas steht die Nummer der unteren zugehrigen Taste (gerade Zahl)
 rcall holmaske             ;Schaltmasken der zugehrigen Taste holen
 ;involvierte Portpins lschen:
 in wl,porta                ;Port A
 and wl,r0                  ;eigene Richtung und Gegenrichtung lschen
 out porta,wl               ;und zurckschreiben
 in wl,portb                ;Port B
 and wl,ausmskb             ;eigene Richtung und Gegenrichtung lschen
 out portb,wl               ;und zurckschreiben
 in wl,portd                ;Port D
 and wl,ausmskd             ;eigene Richtung und Gegenrichtung lschen
 out portd,wl               ;und zurckschreiben
 rjmp mainloop              ;fertig...


TIM0_OVF:               ;ISR alle 256 Takte (256 s)
 in srsk,sreg               ;SREG sichern
 inc zaehler                ;Softwaretimer erhhen
 brne tim0_ovf1             ;alle 65,536ms
 sbr flags,1<<zovf          ;dem Hauptprogramm Zeittakt mitteilen
tim0_ovf1:
 cp zaehler,tout            ;Timeout Impulsabstand erreicht?
 brne isrend                ;nein...
imperr:                 ;Impulserror, neues Telegramm vorbereiten
 mov zsalt,zaehler          ;Zeitstempel holen
 ldi xl,timout              ;Timeout
 add xl,zsalt               ;berechnen und
 out ocr0a,xl               ;setzen
 ldi xl,low(shift0)         ;Start-
 mov shiftl,xl              ;Bitmuster
 ldi xl,high(shift0)        ;ins Shiftregister
 mov shifth,xl              ;schreiben
isrend:                 ;Ende beider ISRs
 out sreg,srsk              ;SREG wiederherstellen
 reti                       ;fertig...

 
exint0:                 ;ISR Impuls vom Empfnger
 in srsk,sreg               ;SREG sichern
 in poko,pind               ;Kopie vom Eingangsport zur Strunterdrckung
 sbrc poko,signal           ;fallende Flanke? - ja...
 rjmp exint01               ;nein, steigende Flanke...
 sbi portd,led1             ;RX-LED ein
 rjmp isrend                ;fertig
exint01:
 cbi portd,led1             ;ja, RX-LED aus
 mov xl,zaehler             ;ja, Impuls gltig, aktuellen Zeitstempel holen
 sub xl,zsalt               ;Differenz zum vorigen Zeitstempel
 mov zsalt,zaehler          ;neuen Zeitstempel merken
 ldi xh,timout              ;Timeout fr Impulsabstanderror 
 mov tout,zaehler           ;auf "jetzt+x" 
 add tout,xh                ;setzen
 cpi xl,17                  ;Impulsabstand unter 0-Wert? (18..22)
 brcs imperr                ;ja, unter 18, Fehler...
 cpi xl,23                  ;Impulsabstand noch 0-Wert? (18..22)
 brcc exint02               ;nein, ber 22, kein 0-Bit mehr
 clc                        ;ja, 0-Bit, Carry lschen
 rjmp shiftin               ;und einschieben
exint02:
 cpi xl,27                  ;Impulsabstand unter 1-Wert? (28..32)
 brcs imperr                ;ja, unter 28, Fehler...
 cpi xl,33                  ;Impulsabstand noch 1-Wert? (28..32)
 brcc imperr                ;nein, ber 32, kein 1-Bit mehr, Error
 sec                        ;ja, 1-Bit, Carry setzen

shiftin:                ;schiebt Carry ein
 rol shiftl                 ;Carry ins untere Shiftregister schieben
 rol shifth                 ;berlauf ins obere Shiftregister schieben
 sbrc shifth,7              ;komplettes Telegramm eingelesen? - ja...
 rjmp isrend                ;nein, fertig...
;Telegramm ist komplett eingelesen
 lsl shiftl                 ;Adressbits
 rol shifth                 ;nach
 lsl shiftl                 ;oben
 rol shifth                 ;schieben,
 lsr shiftl                 ;Command wieder
 lsr shiftl                 ;zurck
 bst shifth,3               ;Togglebit in
 bld shiftl,6               ;Command kopieren
 ldi xl,7                   ;nur die 
 and shifth,xl              ;Adressbits maskieren
 ldi xl,adr0                ;zum Vergleich
 cp shifth,xl               ;stimmt die Adresse?
 breq shiftin1              ;ja...
 rjmp imperr                ;nein, verwerfen...
shiftin1:               ;Adresse stimmt
 cp shiftl,cmd              ;Telegramm gleich dem letzten?
 breq shiftin1a             ;ja...
 clr telez                  ;nein, Telegrammzhler lschen
 mov cmd,shiftl             ;neues Telegramm merken
 rjmp imperr                ;neuen Empfang vorbereiten...
shiftin1a:              ;dasselbe Telegramm wurde wiederholt empfangen
 inc telez                  ;Telegramm-Zhler hoch
 cpi telez,takurz           ;Zhlerstand fr kurzen Tastendruck erreicht?
 brne shiftin1b             ;nein...
 cbr flags,1<<repeat        ;ja, Dauerdruck-Flag erstmal lschen
 rjmp shiftin2              ;weiter...
shiftin1b:              ;kein Kurzdruck-Zhlerstand
 cpi telez,talang           ;Zhlerstand fr langen Tastendruck erreicht?
 brne shiftin3              ;nein...
 dec telez                  ;ja, berlauf vermeiden
 sbr flags,1<<repeat        ;Dauerdruck-Flag setzen
shiftin2:               ;Telegramm stimmt mit vorherigem berein
 sbr flags,1<<neu           ;neues Telegramm melden
shiftin3:               ;Einsprung fr irrelevantes Telegramm
 rjmp imperr                ;nchstes Telegramm vorbereiten...



tastab: ;Tabelle mit selbst definierten Tastennummern (64 Eintrge)
.db 64,64,64,64, 5, 7, 4,64 ;00..07
.db 64,64,64,64,64,64,64, 9 ;08..15
.db 22, 0, 2, 1, 3,10,12,11 ;16..23
.db 13,20,16,18,19,28,17,26 ;24..31
.db 25,34,29,38,23,32,21,30 ;32..39
.db 27,36,27,36,64,64,64,64 ;40..47
.db 24,39,31,35,15,37,64,33 ;48..55
.db 14,64,64,64,64,64, 6, 8 ;56..63
;Wert 64 steht fr leer (von diesem Handsender unbenutzt)
;die Werte 0..39 stehen fr 20 Tastenpaare mit folgender Anordnung:

;    0   2   4   6   8
;    1   3   5   7   9

;   10  12  14  16  18
;   11  13  15  17  19

;   20  22  24  26  28
;   21  23  25  27  29

;   30  32  34  36  38
;   31 (33) 35 (37) 39 

;Eine andere Anordnung htte die Verwaltung der Timeout-Zhler verkompliziert


reltab: ;Tabelle mit Bitmustern fr die Relais (38 Eintrge je 4 Bytes)
;   Aus-Maske  Ein-Maske  Aus-Maske  Ein-Maske
;Port tdddaadd   -dddaadd   bbbbbbbb   bbbbbbbb ;t=Toggle-Flag, a,b,d=Ports
;Bit  -6540110   -6540110   76543210   76543210 ;
.db 0b00000000,0b00000000,0b00000011,0b00000001 ;Taste  0
.db 0b00000000,0b00000000,0b00000011,0b00000010 ;Taste  1
.db 0b00000000,0b00000000,0b00001100,0b00000100 ;Taste  2
.db 0b00000000,0b00000000,0b00001100,0b00001000 ;Taste  3
.db 0b00000000,0b00000000,0b00110000,0b00010000 ;Taste  4
.db 0b00000000,0b00000000,0b00110000,0b00100000 ;Taste  5
.db 0b00000000,0b00000000,0b11000000,0b01000000 ;Taste  6
.db 0b00000000,0b00000000,0b11000000,0b10000000 ;Taste  7
.db 0b00000011,0b00000001,0b00000000,0b00000000 ;Taste  8
.db 0b00000011,0b00000010,0b00000000,0b00000000 ;Taste  9

.db 0b00001100,0b00000100,0b00000000,0b00000000 ;Taste 10
.db 0b00001100,0b00001000,0b00000000,0b00000000 ;Taste 11
.db 0b00110000,0b00010000,0b00000000,0b00000000 ;Taste 12
.db 0b00110000,0b00100000,0b00000000,0b00000000 ;Taste 13
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 14
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 15
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 16
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 17
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 18
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 19
;Port tdddaadd   -dddaadd   bbbbbbbb   bbbbbbbb ;
;Bit  -6540110   -6540110   76543210   76543210 ;
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 20
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 21
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 22
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 23
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 24
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 25
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 26
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 27
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 28
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 29

.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 30
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 31
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 32
.db 0b10000000,0b00010000,0b00000000,0b00000000 ;Taste 33
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 34
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 35
.db 0b00000000,0b00000000,0b11111111,0b01010101 ;Taste 36
.db 0b00000000,0b00000000,0b11111111,0b10101010 ;Taste 37
.db 0b00000000,0b00000000,0b00000000,0b00000000 ;Taste 38
.db 0b10000000,0b01000000,0b00000000,0b00000000 ;Taste 39
;Port tdddaadd   -dddaadd   bbbbbbbb   bbbbbbbb ;
;Bit  -6540110   -6540110   76543210   76543210 ;
;   Aus-Maske  Ein-Maske  Aus-Maske  Ein-Maske

;Ein paar Erklrungen zu den Masken:

;Jede Taste hat Ausschalt- und Einschalt-Masken fr alle verfgbaren Ausgnge.
;Beim Bettigen der Taste werden die Ausgnge ausgeschaltet (falls sie
;eingeschaltet waren), deren Bits in der Aus-Maske gesetzt (1) sind.
;Danach werden die Ausgnge eingeschaltet, deren Bits in der Ein-Maske
;gesetzt (1) sind. Die Zuordnung erfolgt ber die Bezeichnungen a, b und d
;fr die Ports und Ziffern fr die Bitnummern innerhalb der Ports.
;Um Platz zu sparen, wurden den D-Masken die beiden Bits der A-Masken 
;untergeschoben. Die D-Maske nutzt (incl. der A-Bits) die Bits 0 bis 6, die
;B-Maske nutzt alle 8 vorhandenen Bits.

;Bit 7 der Aus-Maske D hat dabei eine Sonderfunktion.
;Ist Bit7 der Aus-Maske D (mit "t" gekennzeichnet) gesetzt (1), dann arbeitet
;diese Taste im Toggelmode ohne die Aus-Masken und den Timeout-Zhler zu 
;benutzen.

;Ist eine Rolladenfahrt bereits aktiv (wird durch Timeout <> 0 ermittelt), so
;wird bei erneutem Tastendrucks einer Taste dieses Tastenpaares die laufende 
;Fahrt abgebrochen.


;(c) Okt.2007 by hannes.lux@gmx.de / http://www.hanneslux.de


/*
Original-Tastencodes (dez.) in Form der Anordnung auf dem Handsender TV400:

17  18  06  62  63
19  20  04  05  15
21  22  56  26  27
23  24  52  30  28
25  16  48  31  29

38  36  32  42  34
39  37  33  43  35

50      51      49

Original-Tastencodes (dez.) in Form der Anordnung auf dem Handsender RCS9082:

17  18  06  62  63
19  20  04  05  15
21  22  56  26  27
23  24  52  30  28
25  16  48  31  29

38  36  32  40  34
39  37  33  41  35

50  55  51  53  49

*/
