;Infrarotempfnger mit Tiny2313 fr Pollin-IR-Bausatz

;Spezialversion fr den Einsatz als AV-Umschalter 1 aus 8.

;Es wird nur auf die Tasten 1 bis 8 reagiert.

;Durch Auswechseln des Pollin-AVRs gegen einen Tiny2313 mit diesem Programm
;entsteht ein vllig neuer IR-Empfnger, der mit derselben Fernbedienung
;angesprochen werden kann.

/*
Tastencodes (dezimal) in Form der Anordnung auf dem Handsender TV400:

17  18  06  62  63
19  20  04  05  15
21  22  56  26  27
23  24  52  30  28
25  16  48  31  29

38  36  32  42  34
39  37  33  43  35

50      51      49

Tastencodes (dezimal) in Form der Anordnung auf dem Handsender RCS9082:

17  18  06  62  63
19  20  04  05  15
21  22  56  26  27
23  24  52  30  28
25  16  48  31  29

38  36  32  40  34
39  37  33  43  35

50  55  51  53  49

*/

.include "tn2313def.inc"

.equ clock=1000000  ;1MHz Taktfrequenz
.equ signal=pd2     ;Signaleingang
.equ led1=pd3       ;Empfangs-LED-Ausgang
.equ shift0=0b1111111111100000  ;Bitmuster fr Empfangs-Start
.equ timout=50      ;Timeout fr Impulsabstand
.equ adr0=7         ;Adresse der Fernbedienung (7)

.dseg               ;SRAM
.org $60            ;ab Adresse 196 (nach mapped-I/O-Bereich)
dauerz:     .byte 8 ;Ausschalt-Zhler fr die Relais

.def srsk=r2        ;SREG-Kopie
.def null=r3        ;immer 0
.def shiftl=r4      ;Empfangsregister L
.def shifth=r5      ;Empfangsregister H
.def zsalt=r6       ;alter Zeitstempel
.def zaehler=r7     ;freilaufender Software-Timer 256 s
.def poko=r8        ;Port-Kopie vom Eingang
.def tout=r9        ;Timeout Impulsabstand
.def cmdalt=r10     ;zuletzt empfangenes Kommando (fr Entprellung)
.def cmd=r11        ;Kommando-Wert nach RECS80-Norm
.def afcz=r12       ;AFC-Zhler (Feinabstimmung)

.def flags=r16      ;einige Flags
    .equ zovf=7         ;Zhler-berlauf fr Zeitstempel
    .equ iovf=6         ;Impulsbreite-berlauf
    .equ neu=5          ;neues Telegramm eingetroffen
    .equ repeat=4       ;Tasten-Dauerdruck liegt an

.def tas=r17        ;Tasten-Wert nach eigener Norm
.def dauer=r18      ;Dauer des Schaltvorgangs am Port
.def kanal=r19      ;momentan aktive Kanalnummer
.def wl=r24         ;Working L
.def wh=r25         ;Working H

.cseg               ;Programmspeicher
.org 0              ;ab Adresse 0
 rjmp RESET         ;Reset Handler
 rjmp exint0        ;External Interrupt0 Handler
 rjmp nix;INT1          ;External Interrupt1 Handler
 rjmp nix;TIM1_CAPT     ;Timer1 Capture Handler
 rjmp nix;TIM1_COMPA    ;Timer1 CompareA Handler
 rjmp nix;TIM1_OVF      ;Timer1 Overflow Handler
 rjmp TIM0_OVF      ;Timer0 Overflow Handler
 rjmp nix;USART0_RXC    ;USART0 RX Complete Handler
 rjmp nix;USART0_DRE    ;USART0,UDR Empty Handler
 rjmp nix;USART0_TXC    ;USART0 TX Complete Handler
 rjmp nix;ANA_COMP      ;Analog Comparator Handler
 rjmp nix;PCINT         ;Pin Change Interrupt
 rjmp nix;TIMER1_COMPB  ;Timer1 Compare B Handler
 rjmp nix;TIMER0_COMPA  ;Timer0 Compare A Handler
 rjmp nix;TIMER0_COMPB  ;Timer0 Compare B Handler
 rjmp nix;USI_START     ;USI Start Handler
 rjmp nix;USI_OVERFLOW  ;USI Overflow Handler
 rjmp nix;EE_READY      ;EEPROM Ready Handler
 rjmp nix;WDT_OVERFLOW  ;Watchdog Overflow Handler
nix:                    ;unerlaubte Interrupts
 rjmp nix                   ;Falle...
 reti                       ;zurck...

reset:                  ;Initialisierung
 ldi wl,low(ramend)         ;Stackpointer
 out spl,wl                 ;initialisieren
 clr null                   ;immer 0
 clr afcz                   ;Zhler Feinabstimmung auf 0
 ldi wl,$ff                 ;alle Pins von
 out ddrb,wl                ;PortB als Ausgang
 out portb,null             ;und L-Pegel
 ldi wl,0b10000111          ;PullUps fr Eingnge
 out portd,wl               ;einschalten (auer LED)
 out porta,wl               ;auch PortA
 com wl                     ;LEDs und Relais 9 und 10 als Ausgang
 out ddrd,wl                ;Jumper und IR-Sensor als Eingang
 out ddra,null              ;PortA (Jumper) als Eingang
 
 out tccr0a,null            ;keine Sonderfunktionen
 ldi wl,1                   ;Vorteiler 1:1
 out tccr0b,wl              ;fr Timer0
 ldi wl,1<<toie0            ;OVF0-Interrupt
 out timsk,wl               ;einschalten
 ldi wl,(1<<se)|(1<<isc00)  ;Sleep-Mode Idle und wechselnde Flanke
 out mcucr,wl               ;fr Int0 vorwhlen
 ldi wl,1<<int0             ;Int0
 out gimsk,wl               ;aktivieren
 ldi flags,0                ;definierte 
 ldi tas,0                  ;Startbedingungen
 out portb,null             ;alle Relais ausschalten
 sei                        ;Interrupts freigeben

mainloop:               ;Hauptschleife
; sbrc flags,zovf            ;Port nach gewisser Zeit
; rcall machaus              ;abschalten
 sbrc flags,neu             ;neues Telegramm eingetroffen? - nein
 rcall neutelda             ;ja, behandeln...
 sleep                      ;bis zum nchsten Interrupt pennen
 rjmp mainloop              ;nochmal...


neutelda:               ;neues Telegramm eingetroffen
 cbr flags,1<<neu           ;Jobflag lschen
 mov tas,cmd                ;Kopie der Kommando-Nummer
 andi tas,63                ;Toggelbit ausblenden
; in 'tas' steht jetzt der Tastencode ohne Toggelbit
 cpi tas, 50                ;On/Off-Taste?
 brne zapp                  ;nein, weiter...
 sbis portb,7               ;ist Gert eingeschaltet? - ja...
 rjmp onoff1                ;nein...
 out portb,null             ;Gert ausschalten
 rjmp neuteldaend           ;fertig...
onoff1:
 clr kanal                  ;Kanal 0
 rjmp setport               ;einschalten
; rjmp neuteldaend           ;fertig...

zapp:                   ;auf Zapp-Tasten prfen 
 sbis portb,7               ;ist Gert eingeschaltet? - ja...
 rjmp direktauswahl         ;nein, Zapp-Tasten ignorieren...
 cpi tas,6                  ;Tastencode 6 (Zapp+) ?
 brne zapp1                 ;nein...
 inc kanal                  ;ja, Kanalnummer erhhen
 cpi kanal,7                ;berlauf?
 brcs zapp0                 ;nein...
 ldi kanal,0                ;ja, von vorn beginnen
zapp0:
 rjmp setport               ;neuen Kanal einschalten
 
zapp1:
 cpi tas,62                 ;Tastencode 62 (Zapp-) ?
 brne direktauswahl         ;nein...
 dec kanal                  ;ja, Kanalnummer vermindern
 sbrc kanal,7               ;Unterlauf? - nein...
 ldi kanal,6                ;ja, von hinten beginnen
zapp2:
 rjmp setport               ;neuen Kanal einschalten

direktauswahl:          ;auf Direktauswahl (Taste 1..7) testen
 subi tas,17                ;ab Tastencode 17
 cpi tas,7                  ;nur 7 Tasten
 brcc neuteldaend           ;diese also nicht...
 mov kanal,tas              ;Kanalnummer direkt zuweisen

setport:
 ldi zl,low(reltab*2)       ;Pointer auf
 ldi zh,high(reltab*2)      ;Relais-Bitmuster
 add zl,kanal               ;Kanalnummer dazu
 adc zh,null                ;bertrag auch
 lpm                        ;Bitmuster nach r0
 out portb,r0               ;an Relaisport ausgeben
 
neuteldaend:            ;Ende der Tastenauswertung
 clr tas                    ;Tastenwert lschen
 ret                        ;fertig



TIM0_OVF:               ;ISR alle 256 Takte (256 s)
 in srsk,sreg               ;SREG sichern
 inc zaehler                ;Softwaretimer erhhen
 mov xl,zaehler             ;Kopie zum Maskieren
 andi xl,3                  ;jedes 4. mal (1ms)
 brne tim0_ovf1             ;jetzt nicht
 sbr flags,1<<zovf          ;dem Hauptprogramm Zeittakt mitteilen
tim0_ovf1:
 cp zaehler,tout            ;Timeout Impulsabstand erreicht?
 brne isrend                ;nein...
imperr:                 ;Impulserror, neues Telegramm vorbereiten
 mov zsalt,zaehler          ;Zeitstempel holen
 ldi xl,timout              ;Timeout
 add xl,zsalt               ;berechnen und
 out ocr0a,xl               ;setzen
 ldi xl,low(shift0)         ;Start-
 mov shiftl,xl              ;Bitmuster
 ldi xl,high(shift0)        ;ins Shiftregister
 mov shifth,xl              ;schreiben
isrend:
 out sreg,srsk              ;SREG wiederherstellen
 reti                       ;fertig...


 
exint0:                 ;ISR Impuls vom Empfnger
 in srsk,sreg               ;SREG sichern
 in poko,pind               ;Kopie vom Eingangsport zur Strunterdrckung
 sbrc poko,signal           ;fallende Flanke? - ja...
 rjmp exint01               ;nein, steigende Flanke...
 sbi portd,led1             ;RX-LED ein
 rjmp isrend                ;fertig
exint01:
 cbi portd,led1             ;ja, RX-LED aus
 mov xl,zaehler             ;ja, Impuls gltig, aktuellen Zeitstempel holen
 sub xl,zsalt               ;Differenz zum vorigen Zeitstempel
 mov zsalt,zaehler          ;neuen Zeitstempel merken
 ldi wh,timout              ;Timeout fr Impulsabstanderror 
 mov tout,zaehler           ;auf "jetzt+x" 
 add tout,wh                ;setzen
 cpi xl,17                  ;Impulsabstand unter 0-Wert? (18..22)
 brcs imperr                ;ja, unter 18, Fehler...
 cpi xl,23                  ;Impulsabstand noch 0-Wert? (18..22)
 brcc exint02               ;nein, ber 22, kein 0-Bit mehr
 clc                        ;ja, 0-Bit, Carry lschen
 rjmp shiftin               ;und einschieben
exint02:
 cpi xl,27                  ;Impulsabstand unter 1-Wert? (28..32)
 brcs imperr                ;ja, unter 28, Fehler...
 cpi xl,33                  ;Impulsabstand noch 1-Wert? (28..32)
 brcc imperr                ;nein, ber 32, kein 1-Bit mehr, Error
 sec                        ;ja, 1-Bit, Carry setzen

shiftin:                ;schiebt Carry ein
 rol shiftl                 ;Carry ins untere Shiftregister schieben
 rol shifth                 ;berlauf ins obere Shiftregister schieben
 sbrc shifth,7              ;komplettes Telegramm eingelesen? - ja...
 rjmp isrend                ;nein, fertig...
;Telegramm ist komplett eingelesen
 lsl shiftl                 ;Adressbits
 rol shifth                 ;nach
 lsl shiftl                 ;oben
 rol shifth                 ;schieben,
 lsr shiftl                 ;Command wieder
 lsr shiftl                 ;zurck
 bst shifth,3               ;Togglebit in
 bld shiftl,6               ;Command kopieren
 ldi xl,7                   ;nur die 
 and shifth,xl              ;Adressbits maskieren
 ldi xl,adr0                ;zum Vergleich
 cp shifth,xl               ;stimmt die Adresse?
 breq shiftin1              ;ja...
 rjmp imperr                ;nein, verwerfen...
shiftin1:               ;Adresse stimmt
 cp shiftl,cmd              ;dieses Telegramm schon erkannt?
 breq shiftin1a             ;ja, verwerfen...
 cp shiftl,cmdalt           ;empfangenes Telegramm stabil?
 breq shiftin2              ;ja...
 mov cmdalt,shiftl          ;nein, neues Telegramm merken,
 rjmp imperr                ;neuen Empfang vorbereiten...
shiftin1a:
 sbr flags,1<<repeat        ;Dauerdruck melden
 rjmp imperr                ;neuen Empfang vorbereiten...
shiftin2:               ;Telegramm stimmt mit vorherigem berein
 mov cmd,shiftl             ;Kommandowert bernehmen
 sbr flags,1<<neu           ;neues Telegramm melden
 rjmp imperr                ;nchstes Telegramm vorbereiten...


reltab: ;Tabelle mit Bitmustern fr die Relais (7 Eintrge)
.db 129,130,132,136,144,160,192,0 ;Bitmuster fr Relais an PortB 

